@@ -58,6 +58,7 @@ fn serialize_node(node: &Node) -> pb::Node {
     node_pb.set_x(node.rooted_coords.0 as u32);
     node_pb.set_y(node.rooted_coords.1 as u32);
     node_pb.set_meta(serialize_meta(&node.meta));
+    node_pb.set_auto_arrange(node.auto_arrange);
     if let Some(ref free_text) = node.free_text {
         node_pb.set_free_text(free_text.to_owned());
     }
@@ -100,7 +101,7 @@ fn deserialize_node(node_pb: &pb::Node) -> Node {
             None
         },
         color: random_fg_color(),
-        auto_arrange: false,
+        auto_arrange: node_pb.get_auto_arrange(),
     }
 }
 
